@@ -277,7 +277,7 @@ typedef struct AuthResult_ {
     unsigned long throttling_bandwidth_ul;
     unsigned long throttling_bandwidth_dl;
     int throttling_ul_changed;
-    int throttling_dl_changed;    
+    int throttling_dl_changed;
 #endif
 #ifdef QUOTAS
     unsigned long long user_quota_size;
@@ -373,7 +373,7 @@ void opendata(void);
 void closedata(void);
 void client_fflush(void);
 void client_printf(const char * const format, ...)
-    __attribute__ ((format(printf, 1, 2)));    
+    __attribute__ ((format(printf, 1, 2)));
 void addreply(const int code, const char * const line, ...)
     __attribute__ ((format(printf, 2, 3)));
 void addreply_noformat(const int code, const char * const line);
@@ -381,7 +381,7 @@ void doreply(void);
 void sighandler(int sig);
 void prevent(char *arg);
 unsigned int daemons(in_port_t server_port);
-void logfile(const int facility, const char *format, ...) 
+void logfile(const int facility, const char *format, ...)
     __attribute__ ((format(printf, 2, 3)));
 char *skip_telnet_controls(const char *str);
 void die(const int err, const int priority, const char * const format, ...)
@@ -468,7 +468,7 @@ extern int opt_a, opt_C, opt_d, opt_F, opt_l, opt_R;
 
 /*
  * Some operating systems (at least Solaris > 2.7 and FreeBSD) have strange
- * troubles with reusing TCP ports, even when SO_REUSEADDR is enabled. 
+ * troubles with reusing TCP ports, even when SO_REUSEADDR is enabled.
  * As a workaround, we try several unassigned privileged ports.
  * The last way is to let the OS assign a port.
  * For egress filtering, you can accept connections from ports <= 20
@@ -479,21 +479,21 @@ extern int opt_a, opt_C, opt_d, opt_F, opt_l, opt_R;
     DEFAULT_FTP_DATA_PORT, 2U, 3U, 4U, 5U, 6U, 10U, 14U, 16U, 0U \
 }
 
-#ifndef MAXPATHLEN
-# ifdef PATH_MAX
-#  define MAXPATHLEN PATH_MAX
+#ifndef PATH_MAX
+# ifdef MAXPATHLEN
+#  define PATH_MAX MAXPATHLEN
 # else
-#  define MAXPATHLEN 65536U
+#  define PATH_MAX 65536U
 Warning: neither PATH_MAX nor MAXPATHLEN were found.
 Remove these lines if you really want to compile the server, but
-the server may be insecure if a wrong value is set here.    
+the server may be insecure if a wrong value is set here.
 # endif
 #endif
-#if (MAXPATHLEN) >= (INT_MAX)
-Your platform has a very large MAXPATHLEN, we should not trust it.
+#if (PATH_MAX) >= (INT_MAX)
+Your platform has a very large PATH_MAX, we should not trust it.
 #endif
 
-#ifndef DEFAULT_MAX_USERS    
+#ifndef DEFAULT_MAX_USERS
 # define DEFAULT_MAX_USERS 50
 #endif
 #ifndef DEFAULT_FTP_DATA_PORT
@@ -504,7 +504,7 @@ Your platform has a very large MAXPATHLEN, we should not trust it.
 # endif
 #endif
 #ifndef MAX_SYSLOG_LINE
-# define MAX_SYSLOG_LINE (MAXPATHLEN + 512U)
+# define MAX_SYSLOG_LINE (PATH_MAX + 512U)
 #endif
 #ifndef DEFAULT_IDLE
 # define DEFAULT_IDLE (15UL * 60UL)
@@ -512,7 +512,7 @@ Your platform has a very large MAXPATHLEN, we should not trust it.
 #ifndef MAX_SITE_IDLE
 # define MAX_SITE_IDLE (42UL * 60UL)
 #endif
-#ifndef DEFAULT_MAX_LS_FILES    
+#ifndef DEFAULT_MAX_LS_FILES
 # define DEFAULT_MAX_LS_FILES 10000U
 #endif
 #ifndef DEFAULT_MAX_LS_DEPTH
@@ -524,7 +524,7 @@ Your platform has a very large MAXPATHLEN, we should not trust it.
 #ifndef MAX_CPU_TIME
 # define MAX_CPU_TIME (4 * 60 * 60)           /* Max allowed CPU time per session */
 #endif
-#ifndef MAX_SESSION_XFER_IDLE    
+#ifndef MAX_SESSION_XFER_IDLE
 # define MAX_SESSION_XFER_IDLE (24 * 60 * 60)   /* Max duration of a transfer */
 #endif
 #ifndef MAX_USER_LENGTH
@@ -537,10 +537,14 @@ Your platform has a very large MAXPATHLEN, we should not trust it.
 # define DEFAULT_FACILITY LOG_LOCAL2
 #endif
 
-#if defined(WITH_LDAP) || defined(WITH_MYSQL) || defined(WITH_PGSQL)
-# define MAX_DATA_SIZE (16 * 1024 * 1024)       /* Max memory usage - SQL/LDAP need more */
-#else
-# define MAX_DATA_SIZE (8 * 1024 * 1024)       /* Max memory usage */
+#ifndef MAX_DATA_SIZE
+# ifdef HAVE_LIBSODIUM
+#  define MAX_DATA_SIZE (24 * 1024 * 1024)
+# elif defined(WITH_LDAP) || defined(WITH_MYSQL) || defined(WITH_PGSQL)
+#  define MAX_DATA_SIZE (16 * 1024 * 1024)       /* Max memory usage - SQL/LDAP need more */
+# else
+#  define MAX_DATA_SIZE (8 * 1024 * 1024)       /* Max memory usage */
+# endif
 #endif
 
 #if CONF_TCP_SO_RCVBUF < 65536
@@ -551,7 +555,7 @@ Your platform has a very large MAXPATHLEN, we should not trust it.
 # undef CONF_TCP_SO_SNDBUF
 # define CONF_TCP_SO_SNDBUF 65536
 #endif
-    
+
 #ifndef DL_MIN_CHUNK_SIZE
 # define DL_MIN_CHUNK_SIZE (8 * 1024)
 #endif
@@ -589,14 +593,14 @@ Your platform has a very large MAXPATHLEN, we should not trust it.
 #endif
 #ifndef UL_MAX_CHUNK_SIZE
 # define UL_MAX_CHUNK_SIZE (512 * 1024UL)
-#endif    
+#endif
 
-#define VHOST_PREFIX_MAX_LEN 64    
+#define VHOST_PREFIX_MAX_LEN 64
 
-#define PUREFTPD_TMPFILE_PREFIX ".pureftpd-"    
+#define PUREFTPD_TMPFILE_PREFIX ".pureftpd-"
 #define ATOMIC_PREFIX_PREFIX PUREFTPD_TMPFILE_PREFIX "upload."
 #define WIN32_ANON_DIR "/ftp"
-    
+
 #define STORAGE_PORT(X)  (*storage_port(&(X)))
 #define STORAGE_PORT6(X) (*storage_port6(&(X)))
 #define STORAGE_SIN_ADDR(X) (storage_sin_addr(&(X))->s_addr)
@@ -664,7 +668,7 @@ Your platform has a very large MAXPATHLEN, we should not trust it.
 #ifdef HAVE_NDIR_H
 # include <ndir.h>
 #endif
-       
+
 #ifdef STAT_MACROS_BROKEN
 # undef S_ISBLK
 # undef S_ISCHR
@@ -677,7 +681,7 @@ Your platform has a very large MAXPATHLEN, we should not trust it.
 # undef S_ISREG
 # undef S_ISSOCK
 #endif                            /* STAT_MACROS_BROKEN.  */
-    
+
 #ifndef S_IFMT
 # define S_IFMT 0170000
 #endif
@@ -709,11 +713,11 @@ Your platform has a very large MAXPATHLEN, we should not trust it.
 #if !defined(S_ISNWK) && defined(S_IFNWK)    /* HP/UX */
 # define S_ISNWK(m) (((m) & S_IFMT) == S_IFNWK)
 #endif
-    
+
 #ifndef S_IEXEC
 # define S_IEXEC S_IXUSR
 #endif
-    
+
 #ifndef S_IXUSR
 # define S_IXUSR S_IEXEC
 #endif
@@ -732,7 +736,7 @@ Your platform has a very large MAXPATHLEN, we should not trust it.
 #else
 # define ISCTRLCODE(X) ((X) == 0x7f || ((unsigned char) (X)) < 32U)
 #endif
-    
+
 #ifndef HAVE_MUNMAP
 # define munmap(A, B) (0)
 #endif
@@ -760,7 +764,7 @@ Your platform has a very large MAXPATHLEN, we should not trust it.
 # endif
 #endif
 
-#if defined(HAVE_DEV_ARANDOM) || defined(HAVE_DEV_URANDOM) || defined(HAVE_DEV_RANDOM)
+#if defined(HAVE_DEV_URANDOM) || defined(HAVE_DEV_RANDOM)
 # define HAVE_RANDOM_DEV 1
 #endif
 
@@ -772,9 +776,9 @@ Your platform has a very large MAXPATHLEN, we should not trust it.
 
 #ifndef offsetof
 # define offsetof(type, member) ((size_t) &((type*) NULL)->member)
-#endif    
-    
-#include "bsd-realpath.h"    
+#endif
+
+#include "bsd-realpath.h"
 #include "fakechroot.h"
-        
+
 #endif
