@@ -28,9 +28,9 @@ GLOBAL(int datafd, -1);                    /* data connection file descriptor */
 GLOBAL0(struct sockaddr_storage ctrlconn);    /* stdin/stdout, for using the same ip number */
 GLOBAL0(signed char v6ready);                    /* IPv6 supported or not */
 GLOBAL0(signed char no_ipv4);                    /* IPv4 disabled or not */
-GLOBAL(const size_t cmdsize, MAXPATHLEN + 16U);
-GLOBAL0(char cmd[MAXPATHLEN + 32U]);        /* command line - about 30 chars for command */
-GLOBAL0(char wd[MAXPATHLEN + 1U]);            /* current working directory */
+GLOBAL(const size_t cmdsize, PATH_MAX + 16U);
+GLOBAL0(char cmd[PATH_MAX + 32U]);        /* command line - about 30 chars for command */
+GLOBAL0(char wd[PATH_MAX + 1U]);            /* current working directory */
 GLOBAL0(char *root_directory);                /* root directory, for chroot'd environments */
 GLOBAL0(signed char loggedin);                    /* != 0 if the user if logged in */
 GLOBAL0(char account[MAX_USER_LENGTH + 1U]);      /* user login */
@@ -83,7 +83,6 @@ GLOBAL(unsigned int maxusers, DEFAULT_MAX_USERS);
 GLOBAL0(unsigned int per_user_max);
 GLOBAL0(unsigned int per_anon_max);
 #endif
-GLOBAL0(int iptropy);
 GLOBAL(volatile int xferfd, -1);
 #ifndef NO_STANDALONE
 GLOBAL0(unsigned int maxip);
@@ -163,8 +162,10 @@ GLOBAL0(signed char be_customer_proof);
 
 #ifdef WITH_TLS
 GLOBAL0(signed char enforce_tls_auth);
-GLOBAL0(int data_protection_level); 
+GLOBAL0(int data_protection_level);
 GLOBAL(const char *tlsciphersuite, NULL);
+GLOBAL0(signed char ssl_disabled);
+GLOBAL0(signed char ssl_verify_client_cert);
 #endif
 
 GLOBAL0(char *atomic_prefix);
@@ -172,7 +173,7 @@ GLOBAL0(char *atomic_prefix);
 #endif
 
 #ifdef WITH_RFC2640
-GLOBAL(signed char utf8, 0);	/* 0: ascii 1: utf-8 */
+GLOBAL(signed char utf8, 0);    /* 0: ascii 1: utf-8 */
 GLOBAL(char *charset_fs, NULL);
 GLOBAL(char *charset_client, NULL);
 GLOBAL(iconv_t iconv_fd_fs2client, (iconv_t) -1);
