@@ -35,11 +35,16 @@
 #define PASSWD_LDAP_SMD5_PREFIX "{smd5}"
 #define PASSWD_LDAP_SHA_PREFIX "{sha}"
 #define PASSWD_LDAP_SSHA_PREFIX "{ssha}"
+#define LDAP_DEFAULT_SCHEME "ldap"
 #define LDAP_DEFAULT_SERVER "localhost"
 #define LDAP_DEFAULT_PORT 389
 #define LDAP_DEFAULT_FILTER "(&(objectClass=posixAccount)(uid=\\L))"
 #define LDAP_DEFAULT_VERSION 3
 
+#define URI_SCHEME_SEPARATOR ":"
+#define URI_AUTHORITY_LEADER "//"
+#define URI_PORT_LEADER ":"
+
 void pw_ldap_parse(const char * const file);
 
 void pw_ldap_check(AuthResult * const result,
