@@ -8,6 +8,10 @@
 #include "pure-pw.h"
 #include "../puredb/src/puredb_read.h"
 
+#ifdef HAVE_LIBSODIUM
+# include <sodium.h>
+#endif
+
 #ifdef WITH_DMALLOC
 # include <dmalloc.h>
 #endif
@@ -21,7 +25,7 @@ void pw_puredb_parse(const char * const file)
     }
     if ((pdb_filename = strdup(file)) == NULL) {
         die_mem();
-    }    
+    }
 }
 
 void pw_puredb_exit(void)
@@ -40,11 +44,11 @@ static char *my_strtok2(char *str, const char delim)
 {
     static char *s;
     static char save;
-    
+
     if (str != NULL) {
         if (*str == 0) {
             return NULL;
-        }        
+        }
         s = str;
         scan:
         while (*s != 0 && *s != delim) {
@@ -52,15 +56,15 @@ static char *my_strtok2(char *str, const char delim)
         }
         save = *s;
         *s = 0;
-        
+
         return str;
     }
-    if (s == NULL || save == 0) {        
+    if (s == NULL || save == 0) {
         return NULL;
     }
     s++;
     str = s;
-    
+
     goto scan;
 }
 
@@ -76,10 +80,10 @@ static int access_ip_match(const struct sockaddr_storage * const sa,
     unsigned long saip;
     const unsigned char *saip_raw;
     char *comapoint;
-        
+
     if (*pattern == 0) {
         return 1;
-    }    
+    }
     if (STORAGE_FAMILY(*sa) != AF_INET) {
         return 0;                      /* TODO: IPv6 */
     }
@@ -93,12 +97,19 @@ static int access_ip_match(const struct sockaddr_storage * const sa,
             if (STORAGE_FAMILY(*sa) != AF_INET || netbits == 0U) {
                 return -1;
             }
-            ip = (ip0 << 24) | (ip1 << 16) | (ip2 << 8) | ip3;
+            ip =
+                ((unsigned long) ip0 << 24) |
+                ((unsigned long) ip1 << 16) |
+                ((unsigned long) ip2 << 8) |
+                (unsigned long) ip3;
             ipcheck_ipdone:
             mask = ~((0x80000000 >> (netbits - 1U)) - 1U);
             saip_raw = (const unsigned char *) &(STORAGE_SIN_ADDR(*sa));
-            saip = (saip_raw[0] << 24) | (saip_raw[1] << 16) |
-                (saip_raw[2] << 8) | saip_raw[3];
+            saip =
+                ((unsigned long) saip_raw[0] << 24) |
+                ((unsigned long) saip_raw[1] << 16) |
+                ((unsigned long) saip_raw[2] << 8) |
+                (unsigned long) saip_raw[3];
             if ((ip & mask) == (saip & mask)) {
                 return 1;
             }
@@ -113,17 +124,21 @@ static int access_ip_match(const struct sockaddr_storage * const sa,
             memset(&hints, 0, sizeof hints);
             hints.ai_family = AF_INET;
             hints.ai_addr = NULL;
-            if ((on = getaddrinfo(pattern, NULL, &hints, &res)) != 0 ||
-                res->ai_family != AF_INET) {
+            if ((on = getaddrinfo(pattern, NULL, &hints, &res)) != 0) {
                 logfile(LOG_WARNING, "puredb: [%s] => [%d]", pattern, on);
+            } else if (res->ai_family != AF_INET) {
+                freeaddrinfo(res);
             } else {
                 const unsigned char * const ip_raw =
                     (const unsigned char *) &
                     (((const struct sockaddr_in *) (void *)
                       (res->ai_addr))->sin_addr.s_addr);
-                
-                ip = (ip_raw[0] << 24) | (ip_raw[1] << 16) |
-                    (ip_raw[2] << 8) | ip_raw[3];
+
+                ip =
+                    ((unsigned long) ip_raw[0] << 24) |
+                    ((unsigned long) ip_raw[1] << 16) |
+                    ((unsigned long) ip_raw[2] << 8) |
+                    (unsigned long) ip_raw[3];
                 netbits = 32U;
                 freeaddrinfo(res);
                 goto ipcheck_ipdone;
@@ -135,7 +150,7 @@ static int access_ip_match(const struct sockaddr_storage * const sa,
         *comapoint = ',';
         pattern = comapoint + 1;
     } while (*pattern != 0);
-        
+
     return 0;
 }
 
@@ -154,7 +169,7 @@ static int access_ip_check(const struct sockaddr_storage * const sa,
         if (access_ip_match(sa, deny) != 0) {
             return -1;
         }
-        return 0;        
+        return 0;
     }
     if (*deny == 0) {
         if (access_ip_match(sa, allow) != 0) {
@@ -171,15 +186,15 @@ static int access_ip_check(const struct sockaddr_storage * const sa,
 static int time_restrictions_check(const char * const restrictions)
 {
     const struct tm *tm;
-    time_t now_t;    
+    time_t now_t;
     unsigned int time_begin, time_end;
     unsigned int now;
-    
+
     if (*restrictions == 0) {
         return 0;
     }
     if (sscanf(restrictions, "%u-%u", &time_begin, &time_end) != 2 ||
-        (now_t = time(NULL)) == (time_t) -1 || 
+        (now_t = time(NULL)) == (time_t) -1 ||
         (tm = localtime(&now_t)) == NULL) {
         return 0;
     }
@@ -192,7 +207,7 @@ static int time_restrictions_check(const char * const restrictions)
     }
     if (now >= time_begin || now <= time_end) {
         return 0;
-    }        
+    }
     return -1;
 }
 
@@ -204,17 +219,29 @@ static int pw_puredb_parseline(char *line, const char * const pwd,
     char *allow_local_ip, *deny_local_ip;
     char *allow_remote_ip, *deny_remote_ip;
     const char *time_restrictions;
-    
+
     if ((line = my_strtok2(line, *PW_LINE_SEP)) == NULL || *line == 0) {   /* pwd */
         return -1;
-    }    
+    }
     {
         const char *crypted;
-        
-        if ((crypted = (const char *) crypt(pwd, line)) == NULL ||
-            strcmp(line, crypted) != 0) {
-            
-            return -1;
+        int         ret = -1;
+
+#ifdef HAVE_LIBSODIUM
+        if (line[0] == '$' && line[1] == '7' && line[2] == '$') {
+            ret = crypto_pwhash_scryptsalsa208sha256_str_verify
+                (line, pwd, strlen(pwd));
+            if (ret != 0) {
+                return -1;
+            }
+        } else
+#endif
+        {
+            ret = - ((crypted = (const char *) crypt(pwd, line)) == NULL ||
+                     strcmp(line, crypted) != 0);
+            if (ret != 0) {
+                return -1;
+            }
         }
     }
     if ((line = my_strtok2(NULL, *PW_LINE_SEP)) == NULL || *line == 0) {   /* uid */
@@ -250,7 +277,7 @@ static int pw_puredb_parseline(char *line, const char * const pwd,
 #endif
     if ((line = my_strtok2(NULL, *PW_LINE_SEP)) == NULL) {   /* bw_dl */
         return 0;
-    } 
+    }
 #ifdef THROTTLING
     if (*line != 0) {
         result->throttling_dl_changed = 1;
@@ -264,10 +291,10 @@ static int pw_puredb_parseline(char *line, const char * const pwd,
     if (*line != 0) {
         result->ratio_upload = (unsigned int) strtoul(line, NULL, 10);
         if (result->ratio_upload > 0U) {
-            result->ratio_ul_changed = 1;     
+            result->ratio_ul_changed = 1;
         }
     }
-#endif    
+#endif
     if ((line = my_strtok2(NULL, *PW_LINE_SEP)) == NULL) {   /* ratio down */
         return 0;
     }
@@ -275,14 +302,14 @@ static int pw_puredb_parseline(char *line, const char * const pwd,
     if (*line != 0) {
         result->ratio_download = (unsigned int) strtoul(line, NULL, 10);
         if (result->ratio_download > 0U) {
-            result->ratio_dl_changed = 1;     
+            result->ratio_dl_changed = 1;
         }
     }
-#endif    
+#endif
     if ((line = my_strtok2(NULL, *PW_LINE_SEP)) == NULL) {   /* max cnx */
         return 0;
     }
-#ifdef PER_USER_LIMITS    
+#ifdef PER_USER_LIMITS
     if (*line != 0) {
     result->per_user_max = (unsigned int) strtoull(line, NULL, 10);
     }
@@ -332,7 +359,7 @@ static int pw_puredb_parseline(char *line, const char * const pwd,
     if (time_restrictions_check(time_restrictions) != 0) {
         return -1;
     }
-    
+
     return 0;
 }
 
@@ -341,11 +368,11 @@ void pw_puredb_check(AuthResult * const result,
                      const struct sockaddr_storage * const sa,
                      const struct sockaddr_storage * const peer)
 {
-    char *line = NULL;    
+    char *line = NULL;
     PureDB db;
     off_t retpos;
     size_t retlen;
-    
+
     result->auth_ok = 0;
     (void) sa;
     (void) peer;
