@@ -26,7 +26,7 @@ extern
 # ifndef IN_TLS_C
 extern
 # endif
-    int tls_cnx_handshaked;
+    int tls_cnx_handshook;
 
 # ifndef IN_TLS_C
 extern
@@ -36,10 +36,10 @@ extern
 # ifndef IN_TLS_C
 extern
 # endif
-    int tls_data_cnx_handshaked;
+    int tls_data_cnx_handshook;
 
 /* The minimal number of bits we accept for a cipher */
-# define MINIMAL_CIPHER_KEY_LEN 40
+# define MINIMAL_CIPHER_STRENGTH_BITS 128
 
 #endif
 #endif
